#include <tulip/Graph.h>
#include <tulip/TlpTools.h>
#include <tulip/PluginLoaderTxt.h>
#include <tulip/PluginLibraryLoader.h>
#include <tulip/LayoutProperty.h>
#include <tulip/SizeProperty.h>
#include <tulip/BooleanProperty.h>
#include <tulip/IntegerProperty.h>

#include <cstdlib>

using namespace std;
using namespace tlp;

Graph *clone(Graph *old_graph) {
    Graph *root_models=old_graph->getSuperGraph();

    //calcul du nom du nouveau modèle
    unsigned num=0;
    root_models->getAttribute<unsigned>("last number", num);     //recherche numéro du dernier modèle créé
    ++num;
    stringstream ss;
    ss << "G" << num;
    Graph *new_graph = old_graph->addCloneSubGraph(ss.str(), true,true);
    root_models->setAttribute<unsigned>("last number", num);

    return new_graph;
}

bool run(Graph* graph, Graph* _rule) {

    Graph *old_graph = graph->getSuperGraph();
    Graph *new_graph = clone(old_graph);

    cerr << "old graph: " << graph->getName() << endl;
    cerr << "new graph: " << new_graph->getName() << endl;

    //copying rule RHS to new_graph
    BooleanProperty newelt(_rule);
    newelt.setAllNodeValue(false);
    newelt.setAllEdgeValue(false);
    //select all elements of RHS
    IntegerProperty* sideprop = _rule->getProperty<IntegerProperty>("RuleSide");
    for(node n: _rule->nodes()) {
        if(sideprop->getNodeValue(n) == 2)
            newelt.setNodeValue(n,true);
    }
    for(edge e: _rule->edges()) {
        if(sideprop->getEdgeValue(e) == 2) {
            newelt.setEdgeValue(e,true);
        }
    }

    //copy RHS to the new graph
    tlp::copyToGraph(new_graph, _rule, &newelt);

    //removing remaining nodes from the LHS instance (edges will be removed automatically)
    new_graph->delNodes(graph->nodes());

    //nettoyage
    if(new_graph->existLocalProperty("Arity"))
        new_graph->delLocalProperty("Arity");
    if(new_graph->existLocalProperty("M"))
        new_graph->delLocalProperty("M");
    if(new_graph->existLocalProperty("N"))
        new_graph->delLocalProperty("N");
    if(new_graph->existLocalProperty("W"))
        new_graph->delLocalProperty("W");
    if(new_graph->existLocalProperty("RuleSide"))
        new_graph->delLocalProperty("RuleSide");

    vector<string> matchingPropertiesList;
    graph->getRoot()->getAttribute<vector<string> >("Matching Properties", matchingPropertiesList);
    for(vector<string>::const_iterator it=matchingPropertiesList.begin();it!=matchingPropertiesList.end();++it) {
        string name(*it + "_isused");
        if(new_graph->existLocalProperty(name)) {
            cerr << "removing property " << name << " from " << new_graph->getName() << endl;
            new_graph->delLocalProperty(name);
        }
    }
    return true;
}

int main(int argc, char* argv[]) {

  if (argc!=2) {
      cerr << "Usage " << argv[0] << " path_to_tlp_graph" << endl;
      return EXIT_FAILURE;
  }
 
  tlp::initTulipLib();
  PluginLoaderTxt loader;
  tlp::PluginLibraryLoader::loadPlugins(&loader);
  
  Graph *g = tlp::loadGraph(argv[1]);
  if(g==nullptr) {
    cerr << "error loading graph" << endl;
    return EXIT_FAILURE;
  }

  Graph *g0 = g->getDescendantGraph("G0");
  Graph *rule = g->getDescendantGraph("rule_1");
  Graph* s;
  while(1) {
      g->push(false);
      forEach(s, g0->getSubGraphs()) {
          cerr << "calling apply rule" << endl;
          if(!run(s, rule)) {
              return false;
          }
      }
      g->pop(false);
  }
  delete g;
  return EXIT_SUCCESS;

}
